@@ -29,15 +29,15 @@ public:
 
     void handleKeyPressEvent(const xcb_key_press_event_t *event);
     void handleKeyReleaseEvent(const xcb_key_release_event_t *event);
+    void handleMappingNotifyEvent(const void *event);
 
     Qt::KeyboardModifiers translateModifiers(int s) const;
-    void updateKeymap(xcb_mapping_notify_event_t *event);
     void updateKeymap();
     QList<int> possibleKeys(const QKeyEvent *e) const;
 
     // when XKEYBOARD not present on the X server
     void updateXKBMods();
-    xkb_mod_mask_t xkbModMask(quint16 state);
+    quint32 xkbModMask(quint16 state);
     void updateXKBStateFromCore(quint16 state);
 #if QT_CONFIG(xinput2)
     void updateXKBStateFromXI(void *modInfo, void *groupInfo);
@@ -47,35 +47,39 @@ public:
     int coreDeviceId() const { return core_device_id; }
     void updateXKBState(xcb_xkb_state_notify_event_t *state);
 #endif
-    void handleStateChanges(xkb_state_component changedComponents);
 
 protected:
-    void handleKeyEvent(xcb_window_t sourceWindow, QEvent::Type type, xcb_keycode_t code,
-                        quint16 state, xcb_timestamp_t time, bool fromSendEvent);
+    void handleKeyEvent(xcb_window_t sourceWindow, QEvent::Type type, xcb_keycode_t code, quint16 state, xcb_timestamp_t time);
 
     void resolveMaskConflicts();
     QString lookupString(struct xkb_state *state, xcb_keycode_t code) const;
-    QString lookupStringNoKeysymTransformations(xkb_keysym_t keysym) const;
-    int keysymToQtKey(xcb_keysym_t keysym, Qt::KeyboardModifiers modifiers,
-                      struct xkb_state *state, xcb_keycode_t code) const;
-
-    typedef QMap<xcb_keysym_t, int> KeysymModifierMap;
-    struct xkb_keymap *keymapFromCore(const KeysymModifierMap &keysymMods);
+    int keysymToQtKey(xcb_keysym_t keysym) const;
+    int keysymToQtKey(xcb_keysym_t keysym, Qt::KeyboardModifiers &modifiers, const QString &text) const;
+    void printKeymapError(const char *error) const;
 
+    void readXKBConfig();
+    void clearXKBConfig();
     // when XKEYBOARD not present on the X server
-    void updateModifiers(const KeysymModifierMap &keysymMods);
-    KeysymModifierMap keysymsToModifiers();
+    void updateModifiers();
     // when XKEYBOARD is present on the X server
     void updateVModMapping();
     void updateVModToRModMapping();
 
     xkb_keysym_t lookupLatinKeysym(xkb_keycode_t keycode) const;
-    void checkForLatinLayout() const;
+    void checkForLatinLayout();
 
 private:
+    void updateXKBStateFromState(struct xkb_state *kb_state, quint16 state);
+
     bool m_config = false;
     xcb_keycode_t m_autorepeat_code = 0;
 
+    struct xkb_context *xkb_context = nullptr;
+    struct xkb_keymap *xkb_keymap = nullptr;
+    struct xkb_state *xkb_state = nullptr;
+    struct xkb_rule_names xkb_names;
+    mutable struct xkb_keymap *latin_keymap = nullptr;
+
     struct _mod_masks {
         uint alt;
         uint altgr;
@@ -87,7 +91,7 @@ private:
     _mod_masks rmod_masks;
 
     // when XKEYBOARD not present on the X server
-    xcb_key_symbols_t *m_key_symbols = nullptr;
+    xcb_key_symbols_t *m_key_symbols;
     struct _xkb_mods {
         xkb_mod_index_t shift;
         xkb_mod_index_t lock;
@@ -104,23 +108,7 @@ private:
     _mod_masks vmod_masks;
     int core_device_id;
 #endif
-
-    struct XKBStateDeleter {
-        void operator()(struct xkb_state *state) const { return xkb_state_unref(state); }
-    };
-    struct XKBKeymapDeleter {
-        void operator()(struct xkb_keymap *keymap) const { return xkb_keymap_unref(keymap); }
-    };
-    struct XKBContextDeleter {
-        void operator()(struct xkb_context *context) const { return xkb_context_unref(context); }
-    };
-    using ScopedXKBState = std::unique_ptr<struct xkb_state, XKBStateDeleter>;
-    using ScopedXKBKeymap = std::unique_ptr<struct xkb_keymap, XKBKeymapDeleter>;
-    using ScopedXKBContext = std::unique_ptr<struct xkb_context, XKBContextDeleter>;
-
-    ScopedXKBState m_xkbState;
-    ScopedXKBKeymap m_xkbKeymap;
-    ScopedXKBContext m_xkbContext;
+    bool m_hasLatinLayout = false;
 };
 
 QT_END_NAMESPACE
