@@ -120,9 +120,6 @@ public:
     QString windowManagerName() const { return m_windowManagerName; }
     bool syncRequestSupported() const { return m_syncRequestSupported; }
 
-    QSurfaceFormat surfaceFormatFor(const QSurfaceFormat &format) const;
-
-    const xcb_visualtype_t *visualForFormat(const QSurfaceFormat &format) const;
     const xcb_visualtype_t *visualForId(xcb_visualid_t) const;
     quint8 depthOfVisual(xcb_visualid_t) const;
 
@@ -137,6 +134,7 @@ public:
     void readXResources();
 
     QFontEngine::HintStyle hintStyle() const { return m_hintStyle; }
+    bool noFontHinting() const { return m_noFontHinting; }
     QFontEngine::SubpixelAntialiasingType subpixelType() const { return m_subpixelType; }
     int antialiasingEnabled() const { return m_antialiasingEnabled; }
 
@@ -172,6 +170,7 @@ private:
     int m_forcedDpi;
     int m_pixelDensity;
     QFontEngine::HintStyle m_hintStyle;
+    bool m_noFontHinting;
     QFontEngine::SubpixelAntialiasingType m_subpixelType;
     int m_antialiasingEnabled;
 };
